@@ -324,11 +324,11 @@ def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
                         mid_distal = mid_distal.flatten()
                         mid_terminal = mid_terminal.flatten()
                         if vessel != 0:
-                            if not tree.boundary.DD[0]((mid_proximal[0],mid_proximal[1],mid_proximal[2],len(tree.boundary.patches)//10)) < 0.1:
+                            if not tree.boundary.DD[0]((mid_proximal[0],mid_proximal[1],mid_proximal[2],2)) < 0.01:
                                 #print('proximal')
                                 include = False
                                 break
-                        val = tree.boundary.DD[0]((mid_distal[0],mid_distal[1],mid_distal[2],len(tree.boundary.patches)//10))
+                        val = tree.boundary.DD[0]((mid_distal[0],mid_distal[1],mid_distal[2],2))
                         if not val < 0.01:
                             #print(val)
                             #plotter = pv.Plotter()
@@ -340,7 +340,7 @@ def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
                             #print('distal')
                             include = False
                             break
-                        if not tree.boundary.DD[0]((mid_terminal[0],mid_terminal[1],mid_terminal[2],len(tree.boundary.patches)//10)) < 0.01:
+                        if not tree.boundary.DD[0]((mid_terminal[0],mid_terminal[1],mid_terminal[2],2)) < 0.01:
                             #print('terminal')
                             include = False
                             break
@@ -370,8 +370,10 @@ def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
             #print("Finite difference: {} Value: {}".format(fd_point,fd_volume[fd_idx]))
             #start = time.time()
             start = perf_counter()
+            assert not np.any(np.isnan(tree.data[:,21])), "Nan found prior to local optimization"
             results = fast_local_function(tree.data,points,terminal,
                                           vessel,gamma,nu,Qterm,Pperm,Pterm)
+            assert not np.any(np.isnan(results[0])), "Nan found after local optimization"
             brute_time = perf_counter()-start
             #truth_results = fast_local_function(tree.data,high_res_points,terminal,
             #                                    vessel,gamma,nu,Qterm,Pperm,Pterm)
@@ -381,6 +383,9 @@ def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
             brute = volume
             idx     = np.argmin(volume)
             bif     = results[5][idx]
+            #print('idx: {}'.format(idx))
+            #print(results[0])
+            #print(results[4])
             #truth_volume = np.pi*(truth_results[0]**lam)*(truth_results[1]**mu)
             #truth_idx = np.argmin(truth_volume)
             #truth_bif = truth_results[5][truth_idx]
@@ -396,6 +401,7 @@ def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
                 start = time.time()
                 data,sub_division_map,sub_division_index = add_bifurcation(tree,vessel,terminal,
                                                                            results,idx,isforest=isforest)
+                assert not np.any(np.isnan(data[:,21])), "Nan found after bifurcation addition"
                 #brute = np.sum(np.pi*data[:,21]**lam*data[:,20]**mu)
                 #print("Brute x: {} Value: {}".format(bif,brute[idx]))
                 #fig = plt.figure()
