@@ -52,27 +52,46 @@ def get():
 
 class tree:
     """
-    Data structure:\n
-    index: 0:2   -> proximal node coordinates \n
-    index: 3:5   -> distal node coordinates \n
-    index: 6:8   -> unit basis U (all) \n
-    index: 9:11  -> unit basis V (all) \n
-    index: 12:14 -> unit basis W (axial direction) (all) \n
-    index: 15,16 -> children (-1 means no child) \n
-    index: 17    -> parent (NA) \n
-    index: 18    -> proximal node index (only real edges) \n
-    index: 19    -> distal node index (only real edges) \n
-    index: 20    -> length (path length) \n
-    index: 21    -> radius (what are the units?) \n
-    index: 22    -> flow (NA) \n
-    index: 23    -> left bifurcation (NA) \n
-    index: 24    -> right bifurcation (NA) \n
-    index: 25    -> reduced resistance (NA) \n
-    index: 26    -> depth (NA) \n
-    index: 27    -> reduced downstream length (NA) \n
-    index: 28    -> root radius scaling factor (same as edge for intermediate) \n
-    index: 29    -> edge that subedge belongs to (if actual edge; self pointer) \n
-    index: 30    -> self identifying index (-1 if intermediate) \n
+    Create a synthetic tree object.
+
+    Parameters
+    ----------
+
+    Attributes
+    ----------
+    data : ndarray
+           This is the contiguous 2d array of vessel data forming the vascular
+           tree. Each row represents a single vessel within the tree. The array
+           has a shape (N,31) where N is the current number of vessel segments
+           within the tree.
+           The following descibe the organization and importance of the column
+           indices for each vessel.
+
+           Column indicies:
+                index: 0:2   -> proximal node coordinates
+                index: 3:5   -> distal node coordinates
+                index: 6:8   -> unit basis U
+                index: 9:11  -> unit basis V
+                index: 12:14 -> unit basis W (axial direction)
+                index: 15,16 -> children (-1 means no child)
+                index: 17    -> parent
+                index: 18    -> proximal node index (only real edges)
+                index: 19    -> distal node index (only real edges)
+                index: 20    -> length (path length)
+                index: 21    -> radius
+                index: 22    -> flow
+                index: 23    -> left bifurcation
+                index: 24    -> right bifurcation
+                index: 25    -> reduced resistance
+                index: 26    -> depth
+                index: 27    -> reduced downstream length
+                index: 28    -> root radius scaling factor
+                index: 29    -> edge that subedge belongs to
+                index: 30    -> self identifying index
+
+    parameters : dict
+          This is a dictionary of values governing the formation of a synthetic
+          tree.
     """
     def __init__(self):
         #self.parameters = {'gamma'   : 3.0,
@@ -198,6 +217,99 @@ class tree:
         #self.sub_division_index = np.array([0])
 
     def add(self,low,high,isforest=False,radius_buffer=0.01,threshold=None,method='L-BFGS-B'):
+        """
+        This method appends a new vessel to the existing tree object and optimizes
+        the bifurcation to minimize for a defined cost function (see cost_function).
+
+        Parameters
+        ----------
+                  low : float
+                       the lower value acceptable within the implicit domain for
+                       which a new terminal location will be considered valid
+                       must be lower than the high value
+
+                  high : float
+                       the higher value acceptable within the implicit domain for
+                       which a new terminal location will be considered valid
+                       must be greater than the low value
+
+                  isforest : boolean
+                       This true/false flag indicates if the tree object being
+                       appended to is part of a larger forest of networks. If so,
+                       vessel appending is deferred until the candidate vessel is
+                       checked against violating remaining networks being built
+                       within the domain. (defualt is false)
+
+                  radius_buffer : float
+                       This is the padding distance for collision checking for the
+                       new vessel. Becuase there are often requirement about the minimum
+                       amount of distance that should exist between any two vessels
+                       it is often necessary to add this padding. (default value is
+                       0.01 units)
+
+                  threshold : float
+                       the minimum threshold euclidean distance that the new termainl
+                       should be placed away from existing vessels in the current tree
+                       (default is None, meaning that the value should be calculated
+                       automatically)
+
+                  method : str
+                       a string argument supplying which type of minimizer to use
+                       during bifurcation optimization. (default is 'L-BFGS-B')
+                       We recommend that you use this since the default cost
+                       function at bifurcations is convex; however, if the a user
+                       defines a non-convex cost function at bifurcations,
+                       performance may be impacted and another (non-newton) method
+                       may be required to sufficiently approach the desired optimum.
+
+                       Available minimizers are the following:
+
+                       'L-BFGS-B' : Limited-Memory Bounded Broyden-Fletcher-Goldfarb-Shanno
+                                    algorithm
+                       'Nelder-Mead' : Nelder-Mead Simplex algorithm
+                       'Powell' : Powell algorithm
+                       'CG' : Conjugate Gradient algorithm
+                       'Newton-CG' : Newton-Conjugate Gradient algorithm
+                       'BFGS' : Broyden-Fletcher-Goldfarb-Shanno algorithm
+                       'TNC' : Truncated Newton algorithm
+                       'COBYLA' : Constrained Optimization BY Linear Approximation algorithm
+                       'SLSQP' : Sequential Least Squares Programming
+                       'dogleg' :  dog-leg trust-region algorithm
+                       'trust-ncg' : Newton conjugate gradient trust-region algorithm
+                       'trust-exact' : nearly exact trust-region algorithm
+                       'trust_krylov' : nearly exact trust-region algorithm;
+                                        only requires matrix vector products
+
+                        See Scipy Optimization for futher documentation on
+                        optimizers shown above.
+
+        Returns
+        -------
+                  vessel : int
+                       the index for the augmented data row containing the appended
+                       vessel data to the vascular tree.
+
+                  data   : ndarray
+                       the augmented 2d array containing all of the relevant values
+                       for the vascular tree with the newly appended vessel and
+                       corresponding bifurcation, flow and radii updates. This
+                       augmented data array will have two more rows than the inital
+                       data array size. Thus array shape updates: (N,31) --> (N+2,31)
+
+                  sub_division_map : list
+                       the augmented list containing downstream vessels relative to each
+                       vessel for fast lookup with the appended new vessel
+
+                  sub_division_index : ndarray
+                       the augmented 1d array mapping vessel index to the corresponding
+                       downstream vessels in the sub_division_map with the appended
+                       vessel
+
+                  threshold : float
+                       the minimum threshold euclidean distance that the new termainl
+                       for which the new terminal point was placed away from existing
+                       vessels in the current tree
+        """
         vessel,data,sub_division_map,sub_division_index,threshold = add_branch(self,low,high,threshold_exponent=0.5,
                                                                      threshold_adjuster=0.75,all_max_attempts=40,
                                                                      max_attemps=3,sampling=20,max_skip=8,
@@ -459,54 +571,63 @@ class tree:
         plotter.set_background('white')
         return plotter
 
-    def export_3d_solid(self,outdir=None,folder="3d_tmp"):
+    def export_3d_solid(self,outdir=None,folder="3d_tmp",watertight=False):
         if outdir is None:
             outdir = os.getcwd()+os.sep+folder
         else:
             outdir = outdir+os.sep+folder
         if not os.path.isdir(outdir):
             os.mkdir(folder)
-        def polyline_from_points(pts,r):
-            poly = pv.PolyData()
-            poly.points = pts
-            cell = np.arange(0,len(pts),dtype=np.int_)
-            cell = np.insert(cell,0,len(pts))
-            poly.lines = cell
-            poly['radius'] = r
-            return poly
-        interp_xyz,interp_r,interp_xyzr = self.export(make=False)
-        vessels = []
-        tets = []
-        surfs = []
-        t_list = np.linspace(0,1,num=1000)
-        for vessel_id in range(len(interp_xyz)):
-            x,y,z = splev(t_list,interp_xyz[vessel_id][0])
-            _,r = splev(t_list,interp_r[vessel_id][0])
-            points = np.zeros((len(t_list),3))
-            points[:,0] = x
-            points[:,1] = y
-            points[:,2] = z
-            polyline = polyline_from_points(points,r)
-            vessel = polyline.tube(radius=min(r),scalars='radius',radius_factor=max(r)/min(r)).triangulate()
-            #vessel = polyline.tube(radius=r[0])
-            vessels.append(vessel)
-            vessel_fix = pymeshfix.MeshFix(remesh_surface(vessel.triangulate(),hausd=0.001,verbosity=0))
-            vessel_fix.repair(verbose=False)
-            surfs.append(vessel_fix.mesh)
-            #tets.append(tetgen.TetGen(vessel_fix.mesh))
-        #for i in tqdm(range(len(tets)),desc="Tetrahedralizing"):
-        #    tets[i].tetrahedralize(minratio=1.2)
-        #surfs = []
-        #for i in tqdm(range(len(tets)),desc="Extracting Surfaces"):
-        #    surfs.append(tets[i].grid.extract_surface().triangulate())
-        #    surf_fix = pymeshfix.MeshFix(surfs[-1])
-        #    surf_fix.repair(verbose=False)
-        #    surfs[-1] = surf_fix.mesh
-        unioned = surfs[0].boolean_union(surfs[1])
-        unioned = unioned.clean()
-        for i in tqdm(range(2,len(surfs)),desc="Unioning"):
-            unioned = unioned.boolean_union(surfs[i])
+        if not watertight:
+            vessels = self.show(show=False)
+            models  = [pv.PolyData(m.GetOutput()) for m in vessels]
+            merge_model   = models[0]
+            for i in range(1,len(models)):
+                merge_model = merge_model.merge(models[i])
+            merge_model.save(outdir+os.sep+"merged_model.vtp")
+            return vessels,merge_model
+        else:
+            def polyline_from_points(pts,r):
+                poly = pv.PolyData()
+                poly.points = pts
+                cell = np.arange(0,len(pts),dtype=np.int_)
+                cell = np.insert(cell,0,len(pts))
+                poly.lines = cell
+                poly['radius'] = r
+                return poly
+            interp_xyz,interp_r,interp_xyzr = self.export(make=False)
+            vessels = []
+            tets = []
+            surfs = []
+            t_list = np.linspace(0,1,num=1000)
+            for vessel_id in range(len(interp_xyz)):
+                x,y,z = splev(t_list,interp_xyz[vessel_id][0])
+                _,r = splev(t_list,interp_r[vessel_id][0])
+                points = np.zeros((len(t_list),3))
+                points[:,0] = x
+                points[:,1] = y
+                points[:,2] = z
+                polyline = polyline_from_points(points,r)
+                vessel = polyline.tube(radius=min(r),scalars='radius',radius_factor=max(r)/min(r)).triangulate()
+                #vessel = polyline.tube(radius=r[0])
+                vessels.append(vessel)
+                vessel_fix = pymeshfix.MeshFix(remesh_surface(vessel.triangulate(),hausd=0.001,verbosity=0))
+                vessel_fix.repair(verbose=False)
+                surfs.append(vessel_fix.mesh)
+                #tets.append(tetgen.TetGen(vessel_fix.mesh))
+            #for i in tqdm(range(len(tets)),desc="Tetrahedralizing"):
+            #    tets[i].tetrahedralize(minratio=1.2)
+            #surfs = []
+            #for i in tqdm(range(len(tets)),desc="Extracting Surfaces"):
+            #    surfs.append(tets[i].grid.extract_surface().triangulate())
+            #    surf_fix = pymeshfix.MeshFix(surfs[-1])
+            #    surf_fix.repair(verbose=False)
+            #    surfs[-1] = surf_fix.mesh
+            unioned = surfs[0].boolean_union(surfs[1])
             unioned = unioned.clean()
+            for i in tqdm(range(2,len(surfs)),desc="Unioning"):
+                unioned = unioned.boolean_union(surfs[i])
+                unioned = unioned.clean()
         unioned.save(outdir+os.sep+"unioned_solid.vtp")
         with open(outdir+os.sep+"simvascular_python_script.py","w") as file:
             file.write(export_3d_only.format(outdir+os.sep+"unioned_solid.vtp"))
@@ -514,8 +635,8 @@ class tree:
 
     def export_0d_simulation(self,steady=True,outdir=None,folder="0d_tmp",number_cardiac_cycles=1,
                       number_time_pts_per_cycle=5,density=1.06,viscosity=0.04,material="olufsen",
-                      olufsen={'k1':0.0,'k2':-22.5267,'k3':1.0e7,'material exponent':1.0,'material pressure':0.0},
-                      linear={'material ehr':1e7,'material pressure':0.0},path_to_0d_solver=None,
+                      olufsen={'k1':0.0,'k2':-22.5267,'k3':1.0e7,'material exponent':2.0,'material pressure':0.0},
+                      linear={'material ehr':1e7,'material pressure':0.0},get_0d_solver=False,path_to_0d_solver=None,
                       viscosity_model='constant',vivo=True,distal_pressure=0):
         """
         This script builds the 0D input file for running 0D simulation.
@@ -532,10 +653,13 @@ class tree:
             outdir = outdir+os.sep+folder
         if not os.path.isdir(outdir):
             os.mkdir(folder)
-        if path_to_0d_solver is None:
-            path_to_0d_solver = locate_0d_solver()
+        if get_0d_solver:
+            if path_to_0d_solver is None:
+                path_to_0d_solver = locate_0d_solver()
+            else:
+                path_to_0d_solver = locate_0d_solver(windows_drive=path_to_0d_solver,linux_drive=path_to_0d_solver)
         else:
-            path_to_0d_solver = locate_0d_solver(windows_drive=path_to_0d_solver,linux_drive=path_to_0d_solver)
+            path_to_0d_solver = None
         input_file = {'description':{'description of case':None,
                                      'analytical results':None},
                       'boundary_conditions':[],
@@ -1150,8 +1274,6 @@ class forest:
             self.boundary.volume = volume
 
     def add(self,number_of_branches,network_id=0,radius_buffer=0.01,exact=True):
-        if self.compete:
-            network_id = -1
         if network_id == -1:
             exit_number = []
             active_networks = list(range(len(self.networks)))
@@ -1167,40 +1289,43 @@ class forest:
                     exit_number.append(self.networks[nid][0].parameters['edge_num'])
         for AN in active_networks:
             for ATr in range(self.trees_per_network[AN]):
-                self.networks[AN][ATr].rng_points,_ = self.boundary.pick(size=40*number_of_branches,homogeneous=True)
-                self.networks[AN][ATr].rng_points = self.networks[AN][ATr].rng_points.tolist()
-                if self.compete:
-                    # new
+                #self.networks[AN][ATr].rng_points,_ = self.boundary.pick(size=40*number_of_branches,homogeneous=True)
+                #self.networks[AN][ATr].rng_points = self.networks[AN][ATr].rng_points.tolist()
+                if not self.compete:
+                    self.networks[AN][ATr].rng_points,_ = self.boundary.pick(size=40*number_of_branches,homogeneous=True)
+                    self.networks[AN][ATr].rng_points = self.networks[AN][ATr].rng_points.tolist()
+                    # new (might be based on a bad assumption??)
                     #self.networks[AN][ATr].rng_points,_ = self.boundary.pick(size=len(self.boundary.tet_verts),homogeneous=True,replacement=False)
                     #self.networks[AN][ATr].rng_points = self.networks[AN][ATr].rng_points.tolist()
-                    for n in tqdm(range(len(self.networks[AN][ATr].rng_points))):
-                        pt = np.array(self.networks[AN][ATr].rng_points.pop(0))
-                        if exact:
-                            other_vessels,distances = close_exact(self.networks[AN][ATr].data,pt)
-                        else:
-                            other_vessels,distances = close(self.networks[AN][ATr].data,pt)
-                        minimum_distance = min(distances)
-                        #if minimum_distance < 4*forest.networks[nid][njd].data[other_vessels[0],21]:
-                        #    continue
-                        retry = False
-                        for idx in active_networks:
-                            for jdx in list(range(self.trees_per_network[idx])):
-                                if idx == AN:
-                                    continue
-                                if exact:
-                                    other_vessels,distances = close_exact(self.networks[idx][jdx].data,pt)
-                                else:
-                                    other_vessels,distances = close(self.networks[idx][jdx].data,pt)
-                                if min(distances) < minimum_distance:
-                                    retry = True
-                                    break
-                            if retry:
-                                break
-                        if retry:
-                            continue
-                        else:
-                            self.networks[AN][ATr].rng_points.insert(-1,pt.tolist())
+                    #for n in tqdm(range(len(self.networks[AN][ATr].rng_points))):
+                    #    pt = np.array(self.networks[AN][ATr].rng_points.pop(0))
+                    #    if exact:
+                    #        other_vessels,distances = close_exact(self.networks[AN][ATr].data,pt)
+                    #    else:
+                    #        other_vessels,distances = close(self.networks[AN][ATr].data,pt)
+                    #    minimum_distance = min(distances)
+                    #    #if minimum_distance < 4*forest.networks[nid][njd].data[other_vessels[0],21]:
+                    #    #    continue
+                    #    retry = False
+                    #    for idx in active_networks:
+                    #        for jdx in list(range(self.trees_per_network[idx])):
+                    #            if idx == AN:
+                    #                continue
+                    #            if exact:
+                    #                other_vessels,distances = close_exact(self.networks[idx][jdx].data,pt)
+                    #            else:
+                    #                other_vessels,distances = close(self.networks[idx][jdx].data,pt)
+                    #            if min(distances) < minimum_distance:
+                    #                retry = True
+                    #                break
+                    #        if retry:
+                    #            break
+                    #    if retry:
+                    #        continue
+                    #    else:
+                    #        self.networks[AN][ATr].rng_points.insert(-1,pt.tolist())
                     #new
+                    #pass
         if not self.compete:
             while len(active_networks) > 0:
                 for nid in active_networks:
